@@ -12,7 +12,7 @@
 ** This file contains routines used for analyzing expressions and
 ** for generating VDBE code that evaluates expressions in SQLite.
 **
-** $Id: expr.c,v 1.14 2002/12/26 16:08:19 matt Exp $
+** $Id: expr.c,v 1.15 2003/01/27 21:50:53 matt Exp $
 */
 #include "sqliteInt.h"
 #include <ctype.h>
@@ -122,7 +122,7 @@ void sqliteExprDelete(Expr *p){
 Expr *sqliteExprDup(Expr *p){
   Expr *pNew;
   if( p==0 ) return 0;
-  pNew = sqliteMalloc( sizeof(*p) );
+  pNew = sqliteMallocRaw( sizeof(*p) );
   if( pNew==0 ) return 0;
   memcpy(pNew, p, sizeof(*pNew));
   if( p->token.z!=0 ){
@@ -488,6 +488,9 @@ int sqliteExprResolveIds(
         pParse->nErr++;
         return 1;
       }
+      if( pExpr->op==TK_COLUMN ){
+        sqliteAuthRead(pParse, pExpr, pTabList, base);
+      }
       break; 
     }
   
@@ -595,6 +598,7 @@ int sqliteExprResolveIds(
       sqliteExprDelete(pRight);
       pExpr->pRight = 0;
       pExpr->op = TK_COLUMN;
+      sqliteAuthRead(pParse, pExpr, pTabList, base);
       break;
     }
 
